@@ -104,6 +104,31 @@ gboolean     g_mount_source_ask_question_finish       (GMountSource
                                                        gboolean                  *aborted,
 						       gint                      *choice_out);
 
+gboolean      g_mount_source_show_processes           (GMountSource              *mount_source,
+						       const char                *message,
+                                                       GArray                    *processes,
+						       const char               **choices,
+						       gint                       n_choices,
+						       gboolean                  *aborted,
+						       gint                      *choice_out);
+
+void          g_mount_source_show_processes_async     (GMountSource              *mount_source,
+						       const char                *message,
+                                                       GArray                    *processes,
+						       const char               **choices,
+						       gint                       n_choices,
+                                                       GAsyncReadyCallback        callback,
+                                                       gpointer                   user_data);
+
+gboolean     g_mount_source_show_processes_finish     (GMountSource              *source,
+                                                       GAsyncResult              *result,
+                                                       gboolean                  *aborted,
+                                                       gint                      *choice_out);
+
+gboolean     g_mount_source_abort                     (GMountSource              *source);
+
+gboolean     g_mount_source_is_dummy                  (GMountSource              *source);
+
 
 const char *  g_mount_source_get_dbus_id              (GMountSource              *mount_source);
 const char *  g_mount_source_get_obj_path             (GMountSource              *mount_source);
