@@ -38,6 +38,11 @@
 #define G_VFS_BACKEND_AFC_MAX_FILE_SIZE G_MAXINT64
 int g_blocksize = 4096; /* assume this is the default block size */
 
+/* AFC_E_INVALID_ARGUMENT was renamed between 0.9.7 and 1.0.0 */
+#ifndef AFC_E_INVALID_ARG
+#define AFC_E_INVALID_ARG AFC_E_INVALID_ARGUMENT
+#endif /* !AFC_E_INVALID_ARG */
+
 struct _GVfsBackendAfc {
   GVfsBackend backend;
 
@@ -62,7 +67,7 @@ static struct afc_error_mapping afc_error_to_g_io_error[] = {
       { AFC_E_READ_ERROR               , G_IO_ERROR_NOT_DIRECTORY },
       { AFC_E_WRITE_ERROR              , G_IO_ERROR_FAILED },
       { AFC_E_UNKNOWN_PACKET_TYPE      , G_IO_ERROR_FAILED },
-      { AFC_E_INVALID_ARGUMENT         , G_IO_ERROR_INVALID_ARGUMENT },
+      { AFC_E_INVALID_ARG              , G_IO_ERROR_INVALID_ARGUMENT },
       { AFC_E_OBJECT_NOT_FOUND         , G_IO_ERROR_NOT_FOUND },
       { AFC_E_OBJECT_IS_DIR            , G_IO_ERROR_IS_DIRECTORY },
       { AFC_E_DIR_NOT_EMPTY            , G_IO_ERROR_NOT_EMPTY },
@@ -381,6 +386,10 @@ g_vfs_backend_afc_mount (GVfsBackend *backend,
     {
       g_vfs_backend_set_icon_name (G_VFS_BACKEND(self), "multimedia-player-apple-ipod-touch");
     }
+  else if ((self->model != NULL) && (strstr(self->model, "iPad") != NULL))
+    {
+      g_vfs_backend_set_icon_name (G_VFS_BACKEND(self), "computer-apple-ipad");
+    }
   else
     {
       g_vfs_backend_set_icon_name (G_VFS_BACKEND(self), "phone-apple-iphone");
@@ -390,12 +399,15 @@ g_vfs_backend_afc_mount (GVfsBackend *backend,
   lockdownd_client_free (lockdown_cli);
 
   /* Add camera item if necessary */
-  dcim_afcinfo = NULL;
-  if (afc_get_file_info (self->afc_cli, "/DCIM", &dcim_afcinfo) == AFC_E_SUCCESS)
-    g_vfs_backend_set_x_content_types (backend, camera_x_content_types);
-  else
-    g_vfs_backend_set_x_content_types (backend, media_player_x_content_types);
-  g_strfreev (dcim_afcinfo);
+  if (virtual_port < 2)
+    {
+      dcim_afcinfo = NULL;
+      if (afc_get_file_info (self->afc_cli, "/DCIM", &dcim_afcinfo) == AFC_E_SUCCESS)
+        g_vfs_backend_set_x_content_types (backend, camera_x_content_types);
+      else
+        g_vfs_backend_set_x_content_types (backend, media_player_x_content_types);
+      g_strfreev (dcim_afcinfo);
+    }
 
   self->connected = TRUE;
   g_vfs_job_succeeded (G_VFS_JOB(job));
@@ -429,6 +441,44 @@ g_vfs_backend_afc_unmount (GVfsBackend *backend,
   g_vfs_job_succeeded (G_VFS_JOB(job));
 }
 
+static gboolean file_get_info (GVfsBackendAfc *backend, const char *path, GFileInfo *info);
+
+static gboolean
+is_directory (GVfsBackendAfc *backend,
+              const char *path)
+{
+  gboolean result = FALSE;
+  GFileInfo *info;
+
+  info = g_file_info_new();
+  if (file_get_info (backend, path, info))
+    {
+      if (g_file_info_get_file_type (info) == G_FILE_TYPE_DIRECTORY)
+        result = TRUE;
+    }
+
+  g_object_unref (info);
+  return result;
+}
+
+static gboolean
+is_regular (GVfsBackendAfc *backend,
+            const char *path)
+{
+  gboolean result = FALSE;
+  GFileInfo *info;
+
+  info = g_file_info_new();
+  if (file_get_info (backend, path, info))
+    {
+      if (g_file_info_get_file_type (info) == G_FILE_TYPE_REGULAR)
+	result = TRUE;
+    }
+
+  g_object_unref (info);
+  return result;
+}
+
 /* Callback to open an existing file for reading. */
 static void
 g_vfs_backend_afc_open_for_read (GVfsBackend *backend,
@@ -441,6 +491,22 @@ g_vfs_backend_afc_open_for_read (GVfsBackend *backend,
   self = G_VFS_BACKEND_AFC(backend);
   g_return_if_fail (self->connected);
 
+  if (is_directory (self, path))
+    {
+      g_vfs_job_failed (G_VFS_JOB (job), G_IO_ERROR,
+                        G_IO_ERROR_IS_DIRECTORY,
+                        _("Can't open directory"));
+      return;
+    }
+
+  if (!is_regular (self, path))
+    {
+      g_vfs_job_failed (G_VFS_JOB (job), G_IO_ERROR,
+                        G_IO_ERROR_NOT_FOUND,
+                        _("File doesn't exist"));
+      return;
+    }
+
   if (G_UNLIKELY(g_vfs_backend_afc_check (afc_file_open (self->afc_cli,
                                                          path, AFC_FOPEN_RDONLY, &fd),
                                           G_VFS_JOB(job))))
@@ -781,6 +847,7 @@ g_vfs_backend_afc_set_info_from_afcinfo (GVfsBackendAfc *self,
           else if (g_str_equal (afcinfo[i+1], "S_IFDIR"))
             {
               type = G_FILE_TYPE_DIRECTORY;
+              content_type = g_strdup ("inode/directory");
             }
           else if (g_str_equal (afcinfo[i+1], "S_IFLNK"))
             {
@@ -825,6 +892,15 @@ g_vfs_backend_afc_set_info_from_afcinfo (GVfsBackendAfc *self,
         }
     }
 
+  if (content_type == NULL)
+    content_type = g_content_type_guess (basename, NULL, 0, NULL);
+  
+  if (content_type)
+    {
+      g_file_info_set_content_type (info, content_type);
+      g_file_info_set_attribute_string (info, G_FILE_ATTRIBUTE_STANDARD_FAST_CONTENT_TYPE, content_type);
+    }
+
   /* and set some additional info */
   g_file_info_set_attribute_uint32 (info, G_FILE_ATTRIBUTE_UNIX_UID, getuid ());
   g_file_info_set_attribute_uint32 (info, G_FILE_ATTRIBUTE_UNIX_GID, getgid ());
@@ -838,13 +914,10 @@ g_vfs_backend_afc_set_info_from_afcinfo (GVfsBackendAfc *self,
     {
       if (type == G_FILE_TYPE_DIRECTORY)
         {
-          content_type = g_strdup ("inode/directory");
           icon = g_themed_icon_new ("folder");
         }
       else
         {
-          if (content_type == NULL)
-            content_type = g_content_type_guess (basename, NULL, 0, NULL);
           if (content_type)
             {
               icon = g_content_type_get_icon (content_type);
@@ -853,12 +926,6 @@ g_vfs_backend_afc_set_info_from_afcinfo (GVfsBackendAfc *self,
             }
         }
 
-      if (content_type)
-        {
-          g_file_info_set_content_type (info, content_type);
-          g_file_info_set_attribute_string (info, G_FILE_ATTRIBUTE_STANDARD_FAST_CONTENT_TYPE, content_type);
-        }
-
       if (icon == NULL)
         icon = g_themed_icon_new ("text-x-generic");
 
@@ -968,6 +1035,36 @@ g_vfs_backend_afc_set_info_from_afcinfo (GVfsBackendAfc *self,
     }
 }
 
+static gboolean
+file_get_info (GVfsBackendAfc *backend,
+               const char *path,
+               GFileInfo *info)
+{
+  char **afcinfo = NULL;
+  const char *basename, *ptr;
+  gboolean result = FALSE;
+
+  g_return_val_if_fail (backend->connected, result);
+  g_return_val_if_fail (info, result);
+
+  if (G_LIKELY(afc_get_file_info (backend->afc_cli, path, &afcinfo) == AFC_E_SUCCESS))
+    {
+      ptr = strrchr (path, '/');
+      if (ptr && ptr[1] != '\0')
+        basename = ptr + 1;
+      else
+        basename = path;
+
+      g_vfs_backend_afc_set_info_from_afcinfo (backend, info, afcinfo, basename, path, NULL, 0);
+      result = TRUE;
+    }
+
+  if (afcinfo)
+    g_strfreev(afcinfo);
+
+  return result;
+}
+
 /* Callback for iterating over a directory. */
 static void
 g_vfs_backend_afc_enumerate (GVfsBackend *backend,
