@@ -209,6 +209,7 @@ g_vfs_ftp_task_acquire_connection (GVfsFtpTask *task)
 
       if (ftp->connections < ftp->max_connections)
         {
+          static GThread *last_thread = NULL;
           /* Save current number of connections here, so we can limit maximum
            * connections later.
            * This is necessary for threading reasons (connections can be
@@ -216,6 +217,7 @@ g_vfs_ftp_task_acquire_connection (GVfsFtpTask *task)
           guint maybe_max_connections = ftp->connections;
 
           ftp->connections++;
+          last_thread = g_thread_self ();
           g_mutex_unlock (ftp->mutex);
           task->conn = g_vfs_ftp_connection_new (ftp->addr, task->cancellable, &task->error);
           if (G_LIKELY (task->conn != NULL))
@@ -227,19 +229,27 @@ g_vfs_ftp_task_acquire_connection (GVfsFtpTask *task)
                 break;
             }
 
-          g_vfs_ftp_task_clear_error (task);
           g_vfs_ftp_connection_free (task->conn);
           task->conn = NULL;
           g_mutex_lock (ftp->mutex);
           ftp->connections--;
-          ftp->max_connections = MIN (ftp->max_connections, maybe_max_connections);
-          if (ftp->max_connections == 0)
+          /* If this value is still equal to our thread it means there were no races 
+           * trying to open connections and the maybe_max_connections value is 
+           * reliable. */
+          if (last_thread == g_thread_self () && 
+              !g_vfs_ftp_task_error_matches (task, G_IO_ERROR, G_IO_ERROR_CANCELLED))
             {
-              g_debug ("no more connections left, exiting...\n");
-              /* FIXME: shut down properly */
-              exit (0);
+              g_print ("maybe: %u, max %u (due to %s)\n", maybe_max_connections, ftp->max_connections, task->error->message);
+              ftp->max_connections = MIN (ftp->max_connections, maybe_max_connections);
+              if (ftp->max_connections == 0)
+                {
+                  g_debug ("no more connections left, exiting...\n");
+                  /* FIXME: shut down properly */
+                  exit (0);
+                }
             }
 
+          g_vfs_ftp_task_clear_error (task);
           continue;
         }
 
@@ -305,8 +315,7 @@ g_vfs_ftp_task_done (GVfsFtpTask *task)
 {
   g_return_if_fail (task != NULL);
 
-  if (task->conn)
-    g_vfs_ftp_task_release_connection (task);
+  g_vfs_ftp_task_release_connection (task);
 
   if (task->job)
     {
@@ -689,33 +698,40 @@ g_vfs_ftp_task_receive (GVfsFtpTask *        task,
         if (flags & G_VFS_FTP_PASS_100)
           break;
         g_vfs_ftp_task_set_error_from_response (task, response);
-        return 0;
+        break;
       case 2:
         if (flags & G_VFS_FTP_FAIL_200)
-          {
-            g_vfs_ftp_task_set_error_from_response (task, response);
-            return 0;
-          }
+          g_vfs_ftp_task_set_error_from_response (task, response);
         break;
       case 3:
         if (flags & G_VFS_FTP_PASS_300)
           break;
         g_vfs_ftp_task_set_error_from_response (task, response);
-        return 0;
+        break;
       case 4:
         g_vfs_ftp_task_set_error_from_response (task, response);
-        return 0;
+        break;
       case 5:
         if ((flags & G_VFS_FTP_PASS_500) ||
             (response == 550 && (flags & G_VFS_FTP_PASS_550)))
           break;
         g_vfs_ftp_task_set_error_from_response (task, response);
-        return 0;
+        break;
       default:
         g_assert_not_reached ();
         break;
     }
 
+  if (g_vfs_ftp_task_is_in_error (task))
+    {
+      if (response != 0 && reply)
+        {
+          g_strfreev (*reply);
+          *reply = NULL;
+        }
+      response = 0;
+    }
+
   return response;
 }
 
@@ -817,13 +833,14 @@ g_vfs_ftp_task_setup_data_connection_pasv (GVfsFtpTask *task, GVfsFtpMethod meth
         	 &port1, &port2) == 6)
        break;
     }
-  g_strfreev (reply);
   if (*s == 0)
     {
+      g_strfreev (reply);
       g_set_error_literal (&task->error, G_IO_ERROR, G_IO_ERROR_FAILED,
         		   _("Invalid reply"));
       return G_VFS_FTP_METHOD_ANY;
     }
+  g_strfreev (reply);
 
   if (method == G_VFS_FTP_METHOD_PASV || method == G_VFS_FTP_METHOD_ANY)
     {
@@ -960,27 +977,71 @@ g_vfs_ftp_task_setup_data_connection_port (GVfsFtpTask *task, GVfsFtpMethod unus
 }
 
 typedef GVfsFtpMethod (* GVfsFtpOpenDataConnectionFunc) (GVfsFtpTask *task, GVfsFtpMethod method);
+typedef struct _GVfsFtpOpenDataConnectionMethod GVfsFtpOpenDataConnectionMethod;
+struct _GVfsFtpOpenDataConnectionMethod {
+    GVfsFtpFeature                required_feature;
+    GSocketFamily                 required_family;
+    GVfsFtpOpenDataConnectionFunc func;
+};
+
+static gboolean
+g_vfs_ftp_task_open_data_connection_method_is_supported (const GVfsFtpOpenDataConnectionMethod *method,
+                                                         GVfsFtpTask *                          task,
+                                                         GSocketFamily                          family)
+{
+  if (method->required_feature &&
+      !g_vfs_backend_ftp_has_feature (task->backend, method->required_feature))
+    return FALSE;
+
+  if (method->required_family != G_SOCKET_FAMILY_INVALID &&
+      method->required_family != family)
+    return FALSE;
+
+  return TRUE;
+}
+
+static GSocketFamily
+g_vfs_ftp_task_get_socket_family (GVfsFtpTask *task)
+{
+  GSocketAddress *addr;
+  GSocketFamily family;
+
+  /* workaround for the task not having a connection yet */
+  if (task->conn == NULL &&
+      g_vfs_ftp_task_send (task, 0, "NOOP") == 0)
+    {
+      g_vfs_ftp_task_clear_error (task);
+      return G_SOCKET_FAMILY_INVALID;
+    }
+
+  addr = g_vfs_ftp_connection_get_address (task->conn, NULL);
+  if (addr == NULL)
+    return G_SOCKET_FAMILY_INVALID;
+
+  family = g_socket_address_get_family (addr);
+  g_object_unref (addr);
+  return family;
+}
 
 static GVfsFtpMethod
 g_vfs_ftp_task_setup_data_connection_any (GVfsFtpTask *task, GVfsFtpMethod unused)
 {
-  static const struct {
-    GVfsFtpFeature required_feature;
-    GVfsFtpOpenDataConnectionFunc func;
-  } funcs_ordered[] = {
-    { G_VFS_FTP_FEATURE_EPSV, g_vfs_ftp_task_setup_data_connection_epsv },
-    { 0,                      g_vfs_ftp_task_setup_data_connection_pasv },
-    { G_VFS_FTP_FEATURE_EPRT, g_vfs_ftp_task_setup_data_connection_eprt },
-    { 0,                      g_vfs_ftp_task_setup_data_connection_port }
+  static const GVfsFtpOpenDataConnectionMethod funcs_ordered[] = {
+    { 0,                      G_SOCKET_FAMILY_IPV4,    g_vfs_ftp_task_setup_data_connection_pasv },
+    { G_VFS_FTP_FEATURE_EPSV, G_SOCKET_FAMILY_INVALID, g_vfs_ftp_task_setup_data_connection_epsv },
+    { 0,                      G_SOCKET_FAMILY_IPV4,    g_vfs_ftp_task_setup_data_connection_port },
+    { G_VFS_FTP_FEATURE_EPRT, G_SOCKET_FAMILY_INVALID, g_vfs_ftp_task_setup_data_connection_eprt }
   };
   GVfsFtpMethod method;
+  GSocketFamily family;
   guint i;
 
+  family = g_vfs_ftp_task_get_socket_family (task);
+
   /* first try all advertised features */
   for (i = 0; i < G_N_ELEMENTS (funcs_ordered); i++)
     {
-      if (funcs_ordered[i].required_feature &&
-          !g_vfs_backend_ftp_has_feature (task->backend, funcs_ordered[i].required_feature))
+      if (!g_vfs_ftp_task_open_data_connection_method_is_supported (&funcs_ordered[i], task, family))
         continue;
       method = funcs_ordered[i].func (task, G_VFS_FTP_METHOD_ANY);
       if (method != G_VFS_FTP_METHOD_ANY)
@@ -992,8 +1053,7 @@ g_vfs_ftp_task_setup_data_connection_any (GVfsFtpTask *task, GVfsFtpMethod unuse
   /* then try if the non-advertised features work */
   for (i = 0; i < G_N_ELEMENTS (funcs_ordered); i++)
     {
-      if (!funcs_ordered[i].required_feature ||
-          g_vfs_backend_ftp_has_feature (task->backend, funcs_ordered[i].required_feature))
+      if (g_vfs_ftp_task_open_data_connection_method_is_supported (&funcs_ordered[i], task, family))
         continue;
       method = funcs_ordered[i].func (task, G_VFS_FTP_METHOD_ANY);
       if (method != G_VFS_FTP_METHOD_ANY)
