@@ -947,10 +947,11 @@ should_drive_be_ignored (GduPool *pool, GduDrive *d, GList *fstab_mount_points)
             ignored = TRUE;
         }
 
-      /* special case for audio discs: don't ignore the drive since we'll create
-       * a cdda:// mount for the drive
+      /* special case for audio and blank discs: don't ignore the drive since we'll create
+       * a cdda:// or burn:// mount for the drive
        */
-      if (gdu_device_is_optical_disc (device) && gdu_device_optical_disc_get_num_audio_tracks (device) > 0)
+      if (gdu_device_is_optical_disc (device) && (gdu_device_optical_disc_get_num_audio_tracks (device) > 0 ||
+                                                  gdu_device_optical_disc_get_is_blank (device)))
         {
           ignored = FALSE;
         }
